"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractWritableShapeFrom = exports.extractProvidedIdColumnNamesFrom = exports.extractAutogeneratedIdColumnNamesFrom = exports.extractIdColumnNamesFrom = exports.extractWritableColumnNamesFrom = exports.extractColumnNamesFrom = exports.extractProvidedIdColumnsFrom = exports.extractAutogeneratedIdColumnsFrom = exports.extractIdColumnsFrom = exports.extractWritableColumnsFrom = exports.extractColumnsFrom = void 0;
const values_1 = require("../expressions/values");
const Column_1 = require("../utils/Column");
function extractColumnsFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, values_1.isValueSource)(value)) {
            result[key] = value;
        }
    }
    return result;
}
exports.extractColumnsFrom = extractColumnsFrom;
function extractWritableColumnsFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            if (!(0, Column_1.__getColumnPrivate)(value).__isComputed) {
                result[key] = value;
            }
        }
    }
    return result;
}
exports.extractWritableColumnsFrom = extractWritableColumnsFrom;
function extractIdColumnsFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isPrimaryKey) {
                result[key] = value;
            }
        }
    }
    return result;
}
exports.extractIdColumnsFrom = extractIdColumnsFrom;
function extractAutogeneratedIdColumnsFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isAutogeneratedPrimaryKey) {
                result[key] = value;
            }
        }
    }
    return result;
}
exports.extractAutogeneratedIdColumnsFrom = extractAutogeneratedIdColumnsFrom;
function extractProvidedIdColumnsFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isPrimaryKey && !columnPrivate.__isAutogeneratedPrimaryKey) {
                result[key] = value;
            }
        }
    }
    return result;
}
exports.extractProvidedIdColumnsFrom = extractProvidedIdColumnsFrom;
function extractColumnNamesFrom(obj, exclude) {
    if (!obj) {
        return [];
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = [];
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, values_1.isValueSource)(value)) {
            result.push(key);
        }
    }
    return result;
}
exports.extractColumnNamesFrom = extractColumnNamesFrom;
function extractWritableColumnNamesFrom(obj, exclude) {
    if (!obj) {
        return [];
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = [];
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            if (!(0, Column_1.__getColumnPrivate)(value).__isComputed) {
                result.push(key);
            }
        }
    }
    return result;
}
exports.extractWritableColumnNamesFrom = extractWritableColumnNamesFrom;
function extractIdColumnNamesFrom(obj, exclude) {
    if (!obj) {
        return [];
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = [];
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isPrimaryKey) {
                result.push(key);
            }
        }
    }
    return result;
}
exports.extractIdColumnNamesFrom = extractIdColumnNamesFrom;
function extractAutogeneratedIdColumnNamesFrom(obj, exclude) {
    if (!obj) {
        return [];
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = [];
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isAutogeneratedPrimaryKey) {
                result.push(key);
            }
        }
    }
    return result;
}
exports.extractAutogeneratedIdColumnNamesFrom = extractAutogeneratedIdColumnNamesFrom;
function extractProvidedIdColumnNamesFrom(obj, exclude) {
    if (!obj) {
        return [];
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = [];
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            const columnPrivate = (0, Column_1.__getColumnPrivate)(value);
            if (columnPrivate.__isPrimaryKey && !columnPrivate.__isAutogeneratedPrimaryKey) {
                result.push(key);
            }
        }
    }
    return result;
}
exports.extractProvidedIdColumnNamesFrom = extractProvidedIdColumnNamesFrom;
function extractWritableShapeFrom(obj, exclude) {
    if (!obj) {
        return obj;
    }
    const ignore = {};
    if (exclude) {
        for (let i = 0, length = exclude.length; i < length; i++) {
            ignore[exclude[i]] = true;
        }
    }
    const result = {};
    for (let key in obj) {
        if (key in ignore) {
            continue;
        }
        const value = obj[key];
        if ((0, Column_1.isColumn)(value)) {
            if (!(0, Column_1.__getColumnPrivate)(value).__isComputed) {
                result[key] = key;
            }
        }
    }
    return result;
}
exports.extractWritableShapeFrom = extractWritableShapeFrom;
